from __future__ import annotations

import os
import re
import subprocess
from datetime import UTC, datetime, timedelta
from pathlib import Path

from .models import EnvInfo, EnvType

_PYTHON_DIR_RE = re.compile(r"^python(?P<version>\d+\.\d+)$")


def _existing_paths(path: Path, candidates: list[str]) -> list[Path]:
    found: list[Path] = []
    for candidate in candidates:
        candidate_path = path / candidate
        if candidate_path.exists():
            found.append(candidate_path)
    return found


def parse_pyvenv_cfg(path: Path) -> dict[str, str]:
    cfg_path = path / "pyvenv.cfg"
    if not cfg_path.is_file():
        return {}

    data: dict[str, str] = {}
    for line in cfg_path.read_text(encoding="utf-8", errors="ignore").splitlines():
        if "=" not in line:
            continue
        key, value = line.split("=", 1)
        data[key.strip().lower()] = value.strip()
    return data


def _find_site_packages_dir(path: Path) -> Path | None:
    direct_candidates = [
        path / "Lib" / "site-packages",
        path / "lib" / "site-packages",
    ]
    for candidate in direct_candidates:
        if candidate.is_dir():
            return candidate

    for lib_root in (path / "lib", path / "Lib"):
        if not lib_root.is_dir():
            continue
        for child in lib_root.iterdir():
            if not child.is_dir():
                continue
            match = _PYTHON_DIR_RE.match(child.name)
            if not match:
                continue
            site_packages = child / "site-packages"
            if site_packages.is_dir():
                return site_packages
    return None


def _extract_python_version(
    path: Path,
    pyvenv_data: dict[str, str],
    *,
    allow_subprocess_probe: bool,
) -> str | None:
    for key in ("version", "version_info", "python-version"):
        value = pyvenv_data.get(key)
        if value:
            return value

    site_packages = _find_site_packages_dir(path)
    if site_packages and site_packages.parent.name.startswith("python"):
        return site_packages.parent.name.removeprefix("python")

    if not allow_subprocess_probe:
        return None

    python_bin = _python_executable(path)
    if python_bin is None:
        return None

    try:
        result = subprocess.run(
            [str(python_bin), "--version"],
            check=False,
            capture_output=True,
            text=True,
            timeout=2,
        )
    except OSError:
        return None

    output = (result.stdout or result.stderr).strip()
    if not output:
        return None

    parts = output.split()
    if len(parts) >= 2:
        return parts[-1]
    return output


def _python_executable(path: Path) -> Path | None:
    for candidate in (path / "bin" / "python", path / "Scripts" / "python.exe"):
        if candidate.is_file():
            return candidate
    return None


def _has_activate_script(path: Path) -> bool:
    return any(
        candidate.is_file()
        for candidate in (path / "bin" / "activate", path / "Scripts" / "activate")
    )


def _calculate_size_bytes(path: Path) -> int:
    total = 0
    for root, _, files in os.walk(path, followlinks=False):
        for filename in files:
            file_path = Path(root) / filename
            try:
                total += file_path.stat().st_size
            except OSError:
                continue
    return total


def _count_packages(path: Path) -> int | None:
    site_packages = _find_site_packages_dir(path)
    if site_packages is None:
        return None

    count = 0
    for entry in site_packages.iterdir():
        if entry.name.endswith(".dist-info") or entry.name.endswith(".egg-info"):
            count += 1
    return count


def list_top_packages(path: Path, limit: int = 10) -> list[str]:
    site_packages = _find_site_packages_dir(path)
    if site_packages is None:
        return []

    packages: list[str] = []
    for entry in site_packages.iterdir():
        if entry.name.endswith(".dist-info"):
            packages.append(entry.name.split("-", 1)[0])
        elif entry.name.endswith(".egg-info"):
            packages.append(entry.name.removesuffix(".egg-info"))

    unique_sorted = sorted(set(packages))
    return unique_sorted[:limit]


def quick_is_environment_dir(path: Path) -> bool:
    if (path / "pyvenv.cfg").is_file():
        return True
    if (path / "conda-meta").is_dir():
        return True
    if _python_executable(path) and _find_site_packages_dir(path):
        return True
    if _has_activate_script(path):
        return True
    return False


def detect_environment(
    path: Path,
    *,
    deep: bool = False,
    stale_days: int = 90,
    include_dotenv: bool = False,
) -> EnvInfo:
    path = path.resolve()
    signals: list[str] = []

    pyvenv_data = parse_pyvenv_cfg(path)
    has_pyvenv_cfg = bool(pyvenv_data)
    if has_pyvenv_cfg:
        signals.append("pyvenv.cfg")

    conda_meta = (path / "conda-meta").is_dir()
    if conda_meta:
        signals.append("conda-meta")

    has_python_bin = _python_executable(path) is not None
    has_site_packages = _find_site_packages_dir(path) is not None
    if has_python_bin:
        signals.append("python-binary")
    if has_site_packages:
        signals.append("site-packages")
    if has_python_bin and has_site_packages:
        signals.append("python+site-packages")

    if _has_activate_script(path):
        signals.append("activate-script")

    definitive_venv = has_pyvenv_cfg or (has_python_bin and has_site_packages)
    strong_venv = _has_activate_script(path) or has_site_packages

    basename = path.name

    env_type = EnvType.UNKNOWN
    if conda_meta:
        env_type = EnvType.CONDA
    elif definitive_venv or strong_venv:
        env_type = EnvType.VENV
    elif basename == ".env":
        env_type = EnvType.DOTENV_DIR

    now = datetime.now(UTC)
    try:
        stat = path.stat()
        created = datetime.fromtimestamp(stat.st_ctime, tz=UTC)
        modified = datetime.fromtimestamp(stat.st_mtime, tz=UTC)
    except OSError:
        created = None
        modified = None

    is_stale = False
    if modified is not None:
        is_stale = modified < (now - timedelta(days=stale_days))

    if env_type == EnvType.DOTENV_DIR and not include_dotenv:
        signals.append("dotenv-dir")

    python_version = _extract_python_version(
        path,
        pyvenv_data,
        allow_subprocess_probe=deep,
    )

    size_bytes = _calculate_size_bytes(path) if deep else None
    package_count = _count_packages(path) if deep else None

    return EnvInfo(
        path=path,
        env_type=env_type,
        python_version=python_version,
        size_bytes=size_bytes,
        created=created,
        modified=modified,
        package_count=package_count,
        is_stale=is_stale,
        has_pyvenv_cfg=has_pyvenv_cfg,
        signals=signals,
    )


def activation_hint(path: Path, env_type: EnvType) -> str:
    if env_type == EnvType.CONDA:
        return f"conda activate {path.name}"

    if (path / "Scripts" / "activate").exists():
        return f"{path / 'Scripts' / 'activate'}"
    return f"source {path / 'bin' / 'activate'}"



from __future__ import annotations

import os
from dataclasses import dataclass, field
from pathlib import Path

from .artifacts import calculate_path_size, match_artifact
from .detector import quick_is_environment_dir
from .models import ArtifactInfo

TARGET_DIR_NAMES = {".env", ".venv", "env", "venv", ".virtualenv", "virtualenv"}
SKIP_DIR_NAMES = {"node_modules", ".git", ".hg", ".svn"}
ALLOWED_HIDDEN = {
    ".env",
    ".venv",
    ".virtualenv",
    ".pyenv",
    ".mypy_cache",
    ".pytest_cache",
    ".ruff_cache",
    ".tox",
    ".nox",
    ".eggs",
    ".ipynb_checkpoints",
}


@dataclass(slots=True)
class ScanDiscovery:
    environments: list[Path]
    artifacts: list[ArtifactInfo] = field(default_factory=list)


def _should_skip(name: str) -> bool:
    if name in SKIP_DIR_NAMES:
        return True
    if name.startswith(".") and name not in ALLOWED_HIDDEN:
        return True
    return False


def scan(
    root: Path,
    max_depth: int = 5,
    *,
    include_artifacts: bool = False,
    deep: bool = False,
) -> ScanDiscovery:
    root = root.resolve()
    found: list[Path] = []
    artifacts: list[ArtifactInfo] = []
    seen: set[Path] = set()
    seen_artifacts: set[Path] = set()

    def walk(current: Path, depth: int) -> None:
        if depth > max_depth:
            return

        try:
            with os.scandir(current) as it:
                entries = list(it)
        except OSError:
            return

        for entry in entries:
            if include_artifacts:
                artifact = match_artifact(entry, current)
                if artifact is not None:
                    if artifact.path not in seen_artifacts:
                        if deep:
                            artifact.size_bytes = calculate_path_size(artifact.path)
                        seen_artifacts.add(artifact.path)
                        artifacts.append(artifact)
                    if entry.is_dir(follow_symlinks=False):
                        continue

            if not entry.is_dir(follow_symlinks=False):
                continue

            name = entry.name
            if _should_skip(name):
                continue

            dir_path = Path(entry.path)
            is_candidate = (
                name in TARGET_DIR_NAMES
                or (dir_path / "pyvenv.cfg").is_file()
                or (dir_path / "conda-meta").is_dir()
            )

            is_env = quick_is_environment_dir(dir_path)
            if is_candidate or is_env:
                resolved = dir_path.resolve()
                if resolved not in seen:
                    seen.add(resolved)
                    found.append(resolved)
                if is_env:
                    continue

            walk(dir_path, depth + 1)

    walk(root, 1)
    return ScanDiscovery(
        environments=sorted(found),
        artifacts=sorted(artifacts, key=lambda item: str(item.path)),
    )



from __future__ import annotations

import os
import shutil
import sys
from pathlib import Path
from typing import Any, TypedDict, cast

import typer

from .artifacts import CAREFUL_NOTES, SAFETY_TEXT
from .models import ArtifactInfo, ArtifactSummary, EnvInfo, SafetyLevel
from .utils import format_age, format_env_display_path, format_size


class DeletionSummary(TypedDict):
    selected_count: int
    deleted_count: int
    failed_count: int
    skipped_count: int
    bytes_freed: int
    would_free_bytes: int
    errors: list[str]
    dry_run: bool


def _abs_no_symlink(path: Path) -> Path:
    return Path(os.path.abspath(path))


def _is_within_root(path: Path, scan_root: Path) -> bool:
    path_abs = _abs_no_symlink(path)
    root_abs = _abs_no_symlink(scan_root)
    try:
        path_abs.relative_to(root_abs)
    except ValueError:
        return False
    return True


def _size_for_deletion(path: Path) -> int:
    try:
        if path.is_symlink():
            return path.lstat().st_size
        if path.is_file():
            return path.stat().st_size
        if not path.is_dir():
            return 0
    except OSError:
        return 0

    total = 0
    for root, _, files in os.walk(path, followlinks=False):
        for filename in files:
            file_path = Path(root) / filename
            try:
                if file_path.is_symlink():
                    total += file_path.lstat().st_size
                else:
                    total += file_path.stat().st_size
            except OSError:
                continue
    return total


def _column_width(display_paths: list[str]) -> int:
    if not display_paths:
        return 30
    max_path_len = max(len(item) for item in display_paths)
    return min(max(max_path_len + 2, 20), 50)


def _table_header(path_width: int) -> str:
    return f"{'Path':<{path_width}} {'Python':<8} {'Size':>6}  {'Age':>5}"


def _choice_text(env: EnvInfo, *, scan_root: Path, path_width: int) -> str:
    display_path = format_env_display_path(env.path, scan_root)
    stale = "  STALE" if env.is_stale else ""
    return (
        f"{display_path:<{path_width}} "
        f"{(env.python_version or '-'): <8} "
        f"{format_size(env.size_bytes):>6}  "
        f"{format_age(env.modified):>5}{stale}"
    )


def _artifact_label(summary: ArtifactSummary) -> str:
    count_label = f"{summary.count} {'dir' if summary.count == 1 else 'dirs'}"
    if summary.pattern in {"*.pyc", "*.pyo", ".coverage"}:
        count_label = f"{summary.count} {'file' if summary.count == 1 else 'files'}"
    size_text = (
        format_size(summary.total_size_bytes) if summary.total_size_bytes > 0 else "-"
    )
    return f"All {summary.pattern:<18} ({count_label:<8}) {size_text:>6}"


def flatten_artifact_summary(summaries: list[ArtifactSummary]) -> list[ArtifactInfo]:
    items: list[ArtifactInfo] = []
    for summary in summaries:
        items.extend(summary.items)
    return items


def _fallback_select(
    environments: list[EnvInfo], *, scan_root: Path, stale_only: bool = False
) -> list[EnvInfo]:
    typer.echo("")
    typer.echo(
        f"Found {len(environments)} environments. "
        "Enter numbers to delete (comma-separated):"
    )
    typer.echo("")
    display_paths = [
        format_env_display_path(env.path, scan_root) for env in environments
    ]
    path_width = _column_width(display_paths)
    typer.echo(f"      {_table_header(path_width)}")
    typer.echo(f"      {'-' * (path_width + 24)}")
    for idx, env in enumerate(environments, start=1):
        marker = "x" if stale_only and env.is_stale else " "
        typer.echo(
            f"  {idx:<3} [{marker}] "
            f"{_choice_text(env, scan_root=scan_root, path_width=path_width)}"
        )

    default = ""
    if stale_only:
        stale_indexes = [
            str(i) for i, env in enumerate(environments, start=1) if env.is_stale
        ]
        default = ",".join(stale_indexes)

    selection_raw = typer.prompt(
        "Select [e.g. 1,3,5]",
        default=default,
        show_default=bool(default),
    ).strip()
    if not selection_raw:
        return []

    selected_indexes: set[int] = set()
    for part in selection_raw.split(","):
        token = part.strip()
        if not token:
            continue
        if not token.isdigit():
            continue
        idx = int(token)
        if 1 <= idx <= len(environments):
            selected_indexes.add(idx - 1)

    return [environments[i] for i in sorted(selected_indexes)]


def interactive_select(
    environments: list[EnvInfo], *, scan_root: Path, stale_only: bool = False
) -> list[EnvInfo]:
    """Display an interactive checklist and return selected environments."""
    if not environments:
        return []

    display_paths = [
        format_env_display_path(env.path, scan_root) for env in environments
    ]
    path_width = _column_width(display_paths)

    is_interactive = sys.stdin.isatty() and sys.stdout.isatty()
    if is_interactive:
        try:
            import questionary

            header = _table_header(path_width)
            rule = "-" * (path_width + 24)
            env_choices = [
                questionary.Choice(
                    title=_choice_text(env, scan_root=scan_root, path_width=path_width),
                    value=index,
                    checked=stale_only and env.is_stale,
                )
                for index, env in enumerate(environments)
            ]
            choices = [
                questionary.Separator(f"  {header}"),
                questionary.Separator(f"  {rule}"),
                *env_choices,
            ]
            selected = questionary.checkbox(
                "Select environments to delete",
                choices=choices,
                instruction="Use ↑↓ to move, Space to toggle, Enter to confirm",
            ).ask()
            if not selected:
                return []
            return [environments[i] for i in selected]
        except ImportError:
            pass
        except KeyboardInterrupt as exc:
            raise typer.Abort() from exc
        except Exception:
            typer.echo(
                "(interactive selection unavailable, using text fallback)", err=True
            )

    return _fallback_select(environments, scan_root=scan_root, stale_only=stale_only)


def interactive_select_with_artifacts(
    environments: list[EnvInfo],
    artifact_summary: list[ArtifactSummary],
    *,
    scan_root: Path,
    stale_only: bool = False,
) -> tuple[list[EnvInfo], list[ArtifactSummary]]:
    if not environments and not artifact_summary:
        return [], []

    display_paths = [
        format_env_display_path(env.path, scan_root) for env in environments
    ]
    path_width = _column_width(display_paths)

    is_interactive = sys.stdin.isatty() and sys.stdout.isatty()
    if is_interactive:
        try:
            import questionary

            choices: list[Any] = []
            header = _table_header(path_width)
            rule = "-" * (path_width + 24)
            if environments:
                choices.extend(
                    [
                        questionary.Separator("  Environments:"),
                        questionary.Separator(f"  {header}"),
                        questionary.Separator(f"  {rule}"),
                    ]
                )
                choices.extend(
                    questionary.Choice(
                        title=_choice_text(
                            env, scan_root=scan_root, path_width=path_width
                        ),
                        value=("env", index),
                        checked=stale_only and env.is_stale,
                    )
                    for index, env in enumerate(environments)
                )

            safety_groups = [
                ("Artifacts (safe to delete):", SafetyLevel.ALWAYS_SAFE),
                ("Artifacts (usually safe):", SafetyLevel.USUALLY_SAFE),
                ("Artifacts (careful - slow to recreate):", SafetyLevel.CAREFUL),
            ]
            for group_title, safety in safety_groups:
                group_items = [
                    item for item in artifact_summary if item.safety == safety
                ]
                if not group_items:
                    continue
                choices.append(questionary.Separator(f"  {group_title}"))
                for index, summary in enumerate(group_items):
                    disabled = "none found" if summary.count == 0 else None
                    choices.append(
                        questionary.Choice(
                            title=_artifact_label(summary),
                            value=("artifact", safety.value, index),
                            checked=False,
                            disabled=disabled,
                        )
                    )

            selected = questionary.checkbox(
                "Select items to delete",
                choices=choices,
                instruction="Use ↑↓ to move, Space to toggle, Enter to confirm",
            ).ask()
            if not selected:
                return [], []

            selected_envs: list[EnvInfo] = []
            selected_artifacts: list[ArtifactSummary] = []
            grouped_by_safety = {
                SafetyLevel.ALWAYS_SAFE: [
                    item
                    for item in artifact_summary
                    if item.safety == SafetyLevel.ALWAYS_SAFE
                ],
                SafetyLevel.USUALLY_SAFE: [
                    item
                    for item in artifact_summary
                    if item.safety == SafetyLevel.USUALLY_SAFE
                ],
                SafetyLevel.CAREFUL: [
                    item
                    for item in artifact_summary
                    if item.safety == SafetyLevel.CAREFUL
                ],
            }
            for tag, *rest in selected:
                if tag == "env":
                    selected_envs.append(environments[rest[0]])
                    continue
                safety = SafetyLevel(rest[0])
                index = rest[1]
                selected_artifacts.append(grouped_by_safety[safety][index])

            return selected_envs, selected_artifacts
        except ImportError:
            pass
        except KeyboardInterrupt as exc:
            raise typer.Abort() from exc
        except Exception:
            typer.echo(
                "(interactive selection unavailable, using text fallback)", err=True
            )

    env_selected = _fallback_select(
        environments, scan_root=scan_root, stale_only=stale_only
    )
    typer.echo("")
    typer.echo("Artifact selections (comma-separated indexes, blank to skip):")
    indexed_artifacts = [item for item in artifact_summary if item.count > 0]
    for idx, summary in enumerate(indexed_artifacts, start=1):
        typer.echo(
            f"  {idx:<3} {_artifact_label(summary)} [{SAFETY_TEXT[summary.safety]}]"
        )
    raw = typer.prompt("Select artifact groups", default="", show_default=False).strip()
    if not raw:
        return env_selected, []
    chosen: set[int] = set()
    for token in raw.split(","):
        token = token.strip()
        if token.isdigit():
            num = int(token)
            if 1 <= num <= len(indexed_artifacts):
                chosen.add(num - 1)
    return env_selected, [indexed_artifacts[i] for i in sorted(chosen)]


def confirm_careful_artifacts(selected: list[ArtifactSummary]) -> bool:
    if not selected:
        return True

    typer.echo("")
    typer.echo("⚠ You selected items that may impact your workflow:")
    typer.echo("")
    for item in selected:
        typer.echo(
            f"  {item.pattern} ({item.count} dirs, {format_size(item.total_size_bytes)})"
        )
        note = CAREFUL_NOTES.get(item.pattern)
        if note:
            typer.echo(f"    {note}")
        typer.echo("")

    response = cast(
        str,
        typer.prompt(
            "Continue with these included? [y/N]", default="N", show_default=False
        ),
    )
    return response.strip().lower() in {"y", "yes"}


def confirm_deletion(
    selected: list[EnvInfo | ArtifactInfo],
    *,
    scan_root: Path,
    dry_run: bool = False,
    skip_confirm: bool = False,
) -> bool:
    """Show deletion summary and require explicit confirmation."""
    typer.echo("")
    typer.echo("⚠ The following items will be PERMANENTLY DELETED:")
    typer.echo("")

    display_paths = [format_env_display_path(item.path, scan_root) for item in selected]
    path_width = _column_width(display_paths)

    total = 0
    for idx, item in enumerate(selected, start=1):
        size = item.size_bytes or 0
        total += size
        typer.echo(
            f"  {idx:<3} "
            f"{format_env_display_path(item.path, scan_root):<{path_width}} "
            f"{format_size(size):>6}"
        )

    typer.echo("")
    typer.echo(f"  Total: {format_size(total)} will be freed")

    if dry_run:
        typer.echo("")
        typer.echo("DRY RUN — no files will be deleted.")
        return False

    if skip_confirm:
        return True

    typed = cast(
        str,
        typer.prompt('Type "delete" to confirm', default="", show_default=False),
    )
    return typed.strip() == "delete"


def delete_environments(
    selected: list[EnvInfo | ArtifactInfo],
    *,
    scan_root: Path,
    dry_run: bool = False,
    dry_run_echo: bool = True,
) -> DeletionSummary:
    """Delete selected environments with path guards and symlink safety."""
    summary: DeletionSummary = {
        "selected_count": len(selected),
        "deleted_count": 0,
        "failed_count": 0,
        "skipped_count": 0,
        "bytes_freed": 0,
        "would_free_bytes": 0,
        "errors": [],
        "dry_run": dry_run,
    }

    for item in selected:
        path = item.path

        if not _is_within_root(path, scan_root):
            warning = f"Skipping outside scan path: {path}"
            typer.echo(f"Warning: {warning}", err=True)
            summary["skipped_count"] += 1
            summary["errors"].append(warning)
            continue

        size = _size_for_deletion(path)
        summary["would_free_bytes"] += size

        if dry_run:
            if dry_run_echo:
                typer.echo(
                    f"[dry-run] Would delete {format_env_display_path(path, scan_root)}"
                )
            continue

        if not path.exists() and not path.is_symlink():
            typer.echo(f"Skipping missing path: {path}")
            summary["skipped_count"] += 1
            continue

        typer.echo(
            f"Deleting {format_env_display_path(path, scan_root)} ...",
            nl=False,
        )
        try:
            if path.is_symlink():
                path.unlink()
            else:
                shutil.rmtree(path)
            summary["deleted_count"] += 1
            summary["bytes_freed"] += size
            typer.echo(" done")
        except PermissionError as exc:
            summary["failed_count"] += 1
            summary["errors"].append(str(exc))
            typer.echo(" failed (permission denied)")
        except OSError as exc:
            summary["failed_count"] += 1
            summary["errors"].append(str(exc))
            typer.echo(" failed")

    return summary


def print_deletion_report(summary: DeletionSummary, *, initial_total: int) -> None:
    """Print post-deletion report in compact box style."""
    remaining = max(0, initial_total - summary["deleted_count"])
    freed_value = (
        summary["would_free_bytes"] if summary["dry_run"] else summary["bytes_freed"]
    )

    typer.echo("─" * 58)
    if summary["dry_run"]:
        typer.echo("  DRY RUN SUMMARY")
    typer.echo(f"  Deleted:   {summary['deleted_count']} items")
    typer.echo(f"  Failed:    {summary['failed_count']}")
    typer.echo(f"  Skipped:   {summary['skipped_count']}")
    typer.echo(f"  Freed:     {format_size(freed_value)}")
    typer.echo(f"  Remaining: {remaining} items")
    typer.echo("─" * 58)



from datetime import UTC, datetime, timedelta
from pathlib import Path

from envoic.models import EnvInfo, EnvType, ScanResult
from envoic.report import bar_chart, format_age, format_list, format_report, format_size


def test_format_size() -> None:
    assert format_size(512) == "512B"
    assert format_size(1024) == "1K"
    assert format_size(1024 * 1024) == "1M"


def test_format_age() -> None:
    now = datetime(2026, 2, 9, tzinfo=UTC)
    assert format_age(now - timedelta(days=12), now=now) == "12d"
    assert format_age(now - timedelta(days=90), now=now) == "3mo"


def test_bar_chart() -> None:
    assert bar_chart(0, 100, width=10) == "[░░░░░░░░░░]"
    assert bar_chart(100, 100, width=10) == "[██████████]"


def test_format_report_structure() -> None:
    env = EnvInfo(
        path=Path("/tmp/project/.venv"),
        env_type=EnvType.VENV,
        python_version="3.12.1",
        size_bytes=1024 * 1024,
        modified=datetime.now(UTC),
    )
    result = ScanResult(
        scan_path=Path("/tmp"),
        scan_depth=5,
        duration_seconds=1.2,
        environments=[env],
        total_size_bytes=1024 * 1024,
        hostname="host-a",
        timestamp=datetime(2026, 2, 9, 12, 0, 0, tzinfo=UTC),
    )

    text = format_report(result)

    assert "ENVOIC - Python Environment Report" in text
    assert "ENVIRONMENTS" in text
    assert "SIZE DISTRIBUTION" in text
    assert "project" in text


def test_format_report_path_modes() -> None:
    env = EnvInfo(
        path=Path("/tmp/project/.venv"),
        env_type=EnvType.VENV,
        python_version="3.12.1",
        size_bytes=1024 * 1024,
        modified=datetime.now(UTC),
    )
    result = ScanResult(
        scan_path=Path("/tmp"),
        scan_depth=5,
        duration_seconds=1.2,
        environments=[env],
        total_size_bytes=1024 * 1024,
        hostname="host-a",
        timestamp=datetime(2026, 2, 9, 12, 0, 0, tzinfo=UTC),
    )

    name_text = format_report(result)
    rel_text = format_report(result, path_mode="relative")
    abs_text = format_report(result, path_mode="absolute")

    assert "project" in name_text
    assert "project" in rel_text
    assert "project/.venv" not in rel_text
    assert "/tmp/project/.venv" in abs_text


def test_format_list_relative_paths() -> None:
    env = EnvInfo(
        path=Path("/tmp/project/.venv"),
        env_type=EnvType.VENV,
        python_version="3.12.1",
    )
    text = format_list([env], path_mode="relative", base_path=Path("/tmp"))
    assert "project" in text
    assert "project/.venv" not in text


def test_format_report_show_artifact_details_false() -> None:
    """Test that artifact details are hidden by default."""
    env = EnvInfo(
        path=Path("/tmp/project/.venv"),
        env_type=EnvType.VENV,
        python_version="3.12.1",
        size_bytes=1024 * 1024,
        modified=datetime.now(UTC),
    )
    result = ScanResult(
        scan_path=Path("/tmp"),
        scan_depth=5,
        duration_seconds=1.2,
        environments=[env],
        total_size_bytes=1024 * 1024,
        hostname="host-a",
        timestamp=datetime(2026, 2, 9, 12, 0, 0, tzinfo=UTC),
    )

    text = format_report(result, show_artifact_details=False)

    assert "ARTIFACT DETAILS" in text
    assert "hidden by default" in text
    assert "run with --show-artifacts" in text.lower() or "run with --artifact" in text.lower()


def test_format_report_show_artifact_details_true() -> None:
    """Test that artifact details are shown when flag is enabled."""
    env = EnvInfo(
        path=Path("/tmp/project/.venv"),
        env_type=EnvType.VENV,
        python_version="3.12.1",
        size_bytes=1024 * 1024,
        modified=datetime.now(UTC),
    )
    result = ScanResult(
        scan_path=Path("/tmp"),
        scan_depth=5,
        duration_seconds=1.2,
        environments=[env],
        total_size_bytes=1024 * 1024,
        hostname="host-a",
        timestamp=datetime(2026, 2, 9, 12, 0, 0, tzinfo=UTC),
    )

    text = format_report(result, show_artifact_details=True)

    assert "ARTIFACTS" in text
    assert "hidden by default" not in text



from __future__ import annotations

import sys
from pathlib import Path


def _ensure_src_on_path() -> None:
    repo_root = Path(__file__).resolve().parent.parent
    src_path = repo_root / "src"
    if str(src_path) not in sys.path:
        sys.path.insert(0, str(src_path))


_ensure_src_on_path()



from __future__ import annotations

from dataclasses import asdict, dataclass, field
from datetime import datetime
from enum import Enum, StrEnum
from pathlib import Path
from typing import Any, Literal, TypedDict, cast, overload


class EnvType(StrEnum):
    VENV = "venv"
    CONDA = "conda"
    DOTENV_DIR = "dotenv_dir"
    UNKNOWN = "unknown"


class SafetyLevel(StrEnum):
    ALWAYS_SAFE = "always_safe"
    USUALLY_SAFE = "usually_safe"
    CAREFUL = "careful"


class ArtifactCategory(StrEnum):
    BYTECODE_CACHE = "bytecode_cache"
    TOOL_CACHE = "tool_cache"
    TEST_ENV = "test_env"
    BUILD_ARTIFACT = "build_artifact"
    COVERAGE_NOTEBOOK = "coverage_notebook"


@dataclass(slots=True)
class EnvInfo:
    path: Path
    env_type: EnvType
    python_version: str | None = None
    size_bytes: int | None = None
    created: datetime | None = None
    modified: datetime | None = None
    package_count: int | None = None
    is_stale: bool = False
    has_pyvenv_cfg: bool = False
    signals: list[str] = field(default_factory=list)


@dataclass(slots=True)
class ScanResult:
    scan_path: Path
    scan_depth: int
    duration_seconds: float
    environments: list[EnvInfo]
    total_size_bytes: int
    hostname: str
    timestamp: datetime
    artifacts: list[ArtifactInfo] = field(default_factory=list)
    artifact_summary: list[ArtifactSummary] = field(default_factory=list)


@dataclass(slots=True)
class ArtifactInfo:
    path: Path
    category: ArtifactCategory
    safety: SafetyLevel
    size_bytes: int | None = None
    pattern_matched: str = ""


@dataclass(slots=True)
class ArtifactSummary:
    """Rolled-up summary per pattern/category."""

    category: ArtifactCategory
    safety: SafetyLevel
    count: int
    total_size_bytes: int
    items: list[ArtifactInfo]
    pattern: str = ""


EnvTypeValue = Literal["venv", "conda", "dotenv_dir", "unknown"]
SafetyLevelValue = Literal["always_safe", "usually_safe", "careful"]
ArtifactCategoryValue = Literal[
    "bytecode_cache",
    "tool_cache",
    "test_env",
    "build_artifact",
    "coverage_notebook",
]


class EnvInfoDict(TypedDict):
    path: str
    env_type: EnvTypeValue
    python_version: str | None
    size_bytes: int | None
    created: str | None
    modified: str | None
    package_count: int | None
    is_stale: bool
    has_pyvenv_cfg: bool
    signals: list[str]


class ArtifactInfoDict(TypedDict):
    path: str
    category: ArtifactCategoryValue
    safety: SafetyLevelValue
    size_bytes: int | None
    pattern_matched: str


class ArtifactSummaryDict(TypedDict):
    category: ArtifactCategoryValue
    safety: SafetyLevelValue
    count: int
    total_size_bytes: int
    items: list[ArtifactInfoDict]
    pattern: str


class ScanResultDict(TypedDict):
    scan_path: str
    scan_depth: int
    duration_seconds: float
    environments: list[EnvInfoDict]
    total_size_bytes: int
    hostname: str
    timestamp: str
    artifacts: list[ArtifactInfoDict]
    artifact_summary: list[ArtifactSummaryDict]


def _serialize_value(value: Any) -> Any:
    if isinstance(value, Path):
        return str(value)
    if isinstance(value, datetime):
        return value.isoformat()
    if isinstance(value, Enum):
        return value.value
    if isinstance(value, list):
        return [_serialize_value(item) for item in value]
    if isinstance(value, dict):
        return {key: _serialize_value(inner) for key, inner in value.items()}
    return value


def _to_serializable_dict(
    data: ScanResult | EnvInfo | ArtifactInfo | ArtifactSummary,
) -> dict[str, Any]:
    return cast(dict[str, Any], _serialize_value(asdict(data)))


@overload
def to_serializable_dict(data: EnvInfo) -> EnvInfoDict: ...


@overload
def to_serializable_dict(data: ScanResult) -> ScanResultDict: ...


@overload
def to_serializable_dict(data: ArtifactInfo) -> ArtifactInfoDict: ...


@overload
def to_serializable_dict(data: ArtifactSummary) -> ArtifactSummaryDict: ...


def to_serializable_dict(
    data: EnvInfo | ScanResult | ArtifactInfo | ArtifactSummary,
) -> EnvInfoDict | ScanResultDict | ArtifactInfoDict | ArtifactSummaryDict:
    serialized = _to_serializable_dict(data)
    return cast(
        EnvInfoDict | ScanResultDict | ArtifactInfoDict | ArtifactSummaryDict,
        serialized,
    )



from __future__ import annotations

import shutil
from pathlib import Path

import pytest
import typer

from envoic.manager import confirm_deletion, delete_environments
from envoic.models import EnvInfo, EnvType
from envoic.utils import format_env_display_path


def _env(path: Path, *, stale: bool = False, size_bytes: int | None = None) -> EnvInfo:
    return EnvInfo(
        path=path,
        env_type=EnvType.VENV,
        is_stale=stale,
        size_bytes=size_bytes,
    )


def test_delete_regular_directory(tmp_path: Path) -> None:
    env_dir = tmp_path / "project" / ".venv"
    env_dir.mkdir(parents=True)
    (env_dir / "file.txt").write_text("abc", encoding="utf-8")

    summary = delete_environments([_env(env_dir)], scan_root=tmp_path)

    assert not env_dir.exists()
    assert summary["deleted_count"] == 1
    assert summary["failed_count"] == 0


def test_delete_symlink_only(tmp_path: Path) -> None:
    target = tmp_path / "target_env"
    target.mkdir()
    (target / "keep.txt").write_text("keep", encoding="utf-8")

    link = tmp_path / "linked_env"
    try:
        link.symlink_to(target, target_is_directory=True)
    except OSError:
        pytest.skip("Symlinks are not supported in this environment")

    summary = delete_environments([_env(link)], scan_root=tmp_path)

    assert not link.exists()
    assert target.exists()
    assert (target / "keep.txt").exists()
    assert summary["deleted_count"] == 1


def test_permission_error_handling(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
) -> None:
    env_dir = tmp_path / "locked" / ".venv"
    env_dir.mkdir(parents=True)

    def _raise_permission(_: Path) -> None:
        raise PermissionError("blocked")

    monkeypatch.setattr(shutil, "rmtree", _raise_permission)
    summary = delete_environments([_env(env_dir)], scan_root=tmp_path)

    assert summary["deleted_count"] == 0
    assert summary["failed_count"] == 1


def test_path_traversal_guard(tmp_path: Path) -> None:
    scan_root = tmp_path / "inside"
    scan_root.mkdir()

    outside = tmp_path / "outside" / ".venv"
    outside.mkdir(parents=True)

    summary = delete_environments([_env(outside)], scan_root=scan_root)

    assert outside.exists()
    assert summary["deleted_count"] == 0
    assert summary["skipped_count"] == 1


def test_dry_run(tmp_path: Path) -> None:
    env_dir = tmp_path / "project" / ".venv"
    env_dir.mkdir(parents=True)
    (env_dir / "f.txt").write_text("abc", encoding="utf-8")

    summary = delete_environments([_env(env_dir)], scan_root=tmp_path, dry_run=True)

    assert env_dir.exists()
    assert summary["deleted_count"] == 0
    assert summary["would_free_bytes"] > 0


def test_confirm_deletion_requires_delete_word(
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    scan_root = Path("/tmp")
    env = _env(scan_root / "project" / ".venv", size_bytes=1024)

    monkeypatch.setattr(typer, "prompt", lambda *_args, **_kwargs: "no")
    assert (
        confirm_deletion([env], scan_root=scan_root, dry_run=False, skip_confirm=False)
        is False
    )

    monkeypatch.setattr(typer, "prompt", lambda *_args, **_kwargs: "delete")
    assert (
        confirm_deletion([env], scan_root=scan_root, dry_run=False, skip_confirm=False)
        is True
    )


def test_format_env_display_path_relative_and_strip(tmp_path: Path) -> None:
    scan_root = tmp_path / "root"
    env_path = scan_root / "apollo-cash" / "app" / ".venv"
    assert format_env_display_path(env_path, scan_root) == "apollo-cash/app"


def test_format_env_display_path_root_edge(tmp_path: Path) -> None:
    scan_root = tmp_path / "root"
    env_path = scan_root / ".venv"
    assert format_env_display_path(env_path, scan_root) == "."



from __future__ import annotations

import os
from collections import defaultdict
from pathlib import Path
from typing import Literal, TypedDict

from .models import ArtifactCategory, ArtifactInfo, ArtifactSummary, SafetyLevel


class ArtifactPattern(TypedDict, total=False):
    name: str
    suffix: str
    type: Literal["dir", "file"]
    category: ArtifactCategory
    safety: SafetyLevel


ARTIFACT_PATTERNS: list[ArtifactPattern] = [
    {
        "name": "__pycache__",
        "type": "dir",
        "category": ArtifactCategory.BYTECODE_CACHE,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
    {
        "suffix": ".pyc",
        "type": "file",
        "category": ArtifactCategory.BYTECODE_CACHE,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
    {
        "suffix": ".pyo",
        "type": "file",
        "category": ArtifactCategory.BYTECODE_CACHE,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
    {
        "name": ".mypy_cache",
        "type": "dir",
        "category": ArtifactCategory.TOOL_CACHE,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
    {
        "name": ".pytest_cache",
        "type": "dir",
        "category": ArtifactCategory.TOOL_CACHE,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
    {
        "name": ".ruff_cache",
        "type": "dir",
        "category": ArtifactCategory.TOOL_CACHE,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
    {
        "name": ".tox",
        "type": "dir",
        "category": ArtifactCategory.TEST_ENV,
        "safety": SafetyLevel.CAREFUL,
    },
    {
        "name": ".nox",
        "type": "dir",
        "category": ArtifactCategory.TEST_ENV,
        "safety": SafetyLevel.CAREFUL,
    },
    {
        "name": "dist",
        "type": "dir",
        "category": ArtifactCategory.BUILD_ARTIFACT,
        "safety": SafetyLevel.USUALLY_SAFE,
    },
    {
        "name": "build",
        "type": "dir",
        "category": ArtifactCategory.BUILD_ARTIFACT,
        "safety": SafetyLevel.USUALLY_SAFE,
    },
    {
        "name": ".eggs",
        "type": "dir",
        "category": ArtifactCategory.BUILD_ARTIFACT,
        "safety": SafetyLevel.USUALLY_SAFE,
    },
    {
        "suffix": ".egg-info",
        "type": "dir",
        "category": ArtifactCategory.BUILD_ARTIFACT,
        "safety": SafetyLevel.CAREFUL,
    },
    {
        "name": ".ipynb_checkpoints",
        "type": "dir",
        "category": ArtifactCategory.COVERAGE_NOTEBOOK,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
    {
        "name": "htmlcov",
        "type": "dir",
        "category": ArtifactCategory.COVERAGE_NOTEBOOK,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
    {
        "name": ".coverage",
        "type": "file",
        "category": ArtifactCategory.COVERAGE_NOTEBOOK,
        "safety": SafetyLevel.ALWAYS_SAFE,
    },
]


PATTERN_ORDER: list[str] = [
    "__pycache__",
    "*.pyc",
    "*.pyo",
    ".mypy_cache",
    ".pytest_cache",
    ".ruff_cache",
    ".tox",
    ".nox",
    "dist",
    "build",
    ".eggs",
    "*.egg-info",
    ".ipynb_checkpoints",
    "htmlcov",
    ".coverage",
]

SAFETY_TEXT: dict[SafetyLevel, str] = {
    SafetyLevel.ALWAYS_SAFE: "safe to delete",
    SafetyLevel.USUALLY_SAFE: "usually safe",
    SafetyLevel.CAREFUL: "careful",
}

CAREFUL_NOTES: dict[str, str] = {
    "*.egg-info": "Editable installs (pip install -e .) depend on these.",
    ".tox": "Tox environments take significant time to recreate.",
    ".nox": "Nox environments take significant time to recreate.",
}


def is_python_project_dir(parent: Path) -> bool:
    return any(
        (parent / marker).exists()
        for marker in ("pyproject.toml", "setup.py", "setup.cfg")
    )


def _pattern_name(pattern: ArtifactPattern) -> str:
    if "name" in pattern:
        return pattern["name"]
    suffix = pattern["suffix"]
    return f"*{suffix}"


def _matches_pattern(entry: os.DirEntry[str], pattern: ArtifactPattern) -> bool:
    name = entry.name
    target_type = pattern["type"]
    if target_type == "dir" and not entry.is_dir(follow_symlinks=False):
        return False
    if target_type == "file" and not entry.is_file(follow_symlinks=False):
        return False

    if "name" in pattern and name != pattern["name"]:
        return False
    if "suffix" in pattern and not name.endswith(pattern["suffix"]):
        return False
    return True


def match_artifact(entry: os.DirEntry[str], parent: Path) -> ArtifactInfo | None:
    for pattern in ARTIFACT_PATTERNS:
        if not _matches_pattern(entry, pattern):
            continue

        pattern_name = _pattern_name(pattern)
        if pattern_name in {"build", "dist"} and not is_python_project_dir(parent):
            continue

        return ArtifactInfo(
            path=Path(entry.path).resolve(),
            category=pattern["category"],
            safety=pattern["safety"],
            pattern_matched=pattern_name,
        )
    return None


def calculate_path_size(path: Path) -> int:
    try:
        if path.is_symlink():
            return path.lstat().st_size
        if path.is_file():
            return path.stat().st_size
    except OSError:
        return 0

    total = 0
    for root, _, files in os.walk(path, followlinks=False):
        for filename in files:
            file_path = Path(root) / filename
            try:
                if file_path.is_symlink():
                    total += file_path.lstat().st_size
                else:
                    total += file_path.stat().st_size
            except OSError:
                continue
    return total


def summarize_artifacts(artifacts: list[ArtifactInfo]) -> list[ArtifactSummary]:
    grouped: dict[tuple[ArtifactCategory, SafetyLevel, str], list[ArtifactInfo]] = (
        defaultdict(list)
    )
    for item in artifacts:
        key = (item.category, item.safety, item.pattern_matched)
        grouped[key].append(item)

    pattern_position = {pattern: idx for idx, pattern in enumerate(PATTERN_ORDER)}
    summaries: list[ArtifactSummary] = []
    for (category, safety, pattern), items in grouped.items():
        total = sum(item.size_bytes or 0 for item in items)
        summaries.append(
            ArtifactSummary(
                category=category,
                safety=safety,
                count=len(items),
                total_size_bytes=total,
                items=sorted(items, key=lambda item: str(item.path)),
                pattern=pattern,
            )
        )
    return sorted(
        summaries,
        key=lambda item: (
            pattern_position.get(item.pattern, 999),
            item.category.value,
            item.pattern,
        ),
    )


def summarize_with_empty_patterns(
    artifacts: list[ArtifactInfo],
) -> list[ArtifactSummary]:
    by_pattern: dict[str, ArtifactSummary] = {
        summary.pattern: summary for summary in summarize_artifacts(artifacts)
    }
    pattern_by_name = {_pattern_name(item): item for item in ARTIFACT_PATTERNS}
    summaries: list[ArtifactSummary] = []
    for pattern_name in PATTERN_ORDER:
        existing = by_pattern.get(pattern_name)
        if existing is not None:
            summaries.append(existing)
            continue
        pattern = pattern_by_name.get(pattern_name)
        if pattern is None:
            continue
        summaries.append(
            ArtifactSummary(
                category=pattern["category"],
                safety=pattern["safety"],
                count=0,
                total_size_bytes=0,
                items=[],
                pattern=pattern_name,
            )
        )
    return summaries



from __future__ import annotations

from datetime import UTC, datetime
from pathlib import Path

VENV_DIR_NAMES = {".env", ".venv", "env", "venv", ".virtualenv", "virtualenv"}


def format_env_display_path(env_path: Path, scan_root: Path) -> str:
    """Return env path relative to root and without trailing venv folder name."""
    try:
        rel = env_path.relative_to(scan_root)
    except ValueError:
        text = str(env_path)
        home = str(Path.home())
        if text.startswith(home):
            return "~" + text[len(home) :]
        return text

    if rel.name in VENV_DIR_NAMES:
        rel = rel.parent

    if str(rel) in {"", "."}:
        return "."
    return str(rel)


def format_size(num_bytes: int | None) -> str:
    if num_bytes is None:
        return "-"
    if num_bytes < 1024:
        return f"{num_bytes}B"

    units = ["K", "M", "G", "T"]
    value = float(num_bytes)
    for unit in units:
        value /= 1024.0
        if value < 1024 or unit == units[-1]:
            if value >= 100:
                return f"{value:.0f}{unit}"
            if value >= 10:
                return f"{value:.1f}{unit}".replace(".0", "")
            return f"{value:.1f}{unit}".replace(".0", "")
    return f"{num_bytes}B"


def format_age(moment: datetime | None, now: datetime | None = None) -> str:
    if moment is None:
        return "-"
    if now is None:
        now = datetime.now(UTC)

    delta = now - moment.astimezone(UTC)
    days = max(0, delta.days)
    if days < 30:
        return f"{days}d"
    if days < 365:
        return f"{days // 30}mo"
    return f"{days // 365}y"


def bar_chart(value: int, max_value: int, width: int = 24) -> str:
    if max_value <= 0:
        return "[" + ("░" * width) + "]"
    filled = int(round((value / max_value) * width))
    filled = max(0, min(width, filled))
    return "[" + ("█" * filled) + ("░" * (width - filled)) + "]"


def shorten_path(path: Path, max_len: int = 36) -> str:
    text = str(path)
    home = str(Path.home())
    if text.startswith(home):
        text = "~" + text[len(home) :]

    if len(text) <= max_len:
        return text

    keep = max_len - 3
    prefix = keep // 2
    suffix = keep - prefix
    return f"{text[:prefix]}...{text[-suffix:]}"



from __future__ import annotations

from pathlib import Path
from typing import Literal

from .artifacts import CAREFUL_NOTES, SAFETY_TEXT
from .models import ArtifactSummary, EnvInfo, SafetyLevel, ScanResult
from .utils import (
    VENV_DIR_NAMES,
    bar_chart,
    format_age,
    format_env_display_path,
    format_size,
    shorten_path,
)

REPORT_WIDTH = 58
PathMode = Literal["name", "relative", "absolute"]


def _box_top(width: int = REPORT_WIDTH) -> str:
    return "┌" + ("─" * width) + "┐"


def _box_mid(width: int = REPORT_WIDTH) -> str:
    return "├" + ("─" * width) + "┤"


def _box_bottom(width: int = REPORT_WIDTH) -> str:
    return "└" + ("─" * width) + "┘"


def box_line(text: str, width: int = REPORT_WIDTH) -> str:
    clipped = text[:width]
    return f"│{clipped:<{width}}│"


def _row(label: str, value: str, width: int = REPORT_WIDTH) -> str:
    left = f"  {label:<12}"
    right_width = max(0, width - len(left) - 2)
    value = value[:right_width]
    return box_line(f"{left}{value:>{right_width}}  ", width=width)


def _table_header() -> str:
    return f"  {'#':<3} {'Path':<30} {'Python':<8} {'Size':>6} {'Age':>5}"


def _truncate_text(text: str, width: int) -> str:
    if len(text) <= width:
        return text
    keep = width - 3
    prefix = keep // 2
    suffix = keep - prefix
    return f"{text[:prefix]}...{text[-suffix:]}"


def _environment_label(
    env_path: Path,
    width: int,
    *,
    path_mode: PathMode,
    base_path: Path | None = None,
) -> str:
    if path_mode == "absolute":
        return _truncate_text(str(env_path), width)

    if path_mode == "relative":
        if base_path is not None:
            try:
                return _truncate_text(
                    format_env_display_path(env_path, base_path), width
                )
            except (OSError, ValueError):
                pass
        return _truncate_text(str(env_path), width)

    name = env_path.name
    if name in VENV_DIR_NAMES and env_path.parent.name:
        name = env_path.parent.name
    return _truncate_text(name, width)


def _table_row(
    index: int,
    env: EnvInfo,
    *,
    path_mode: PathMode,
    base_path: Path | None = None,
) -> str:
    stale = " STALE" if env.is_stale else ""
    return (
        f"  {index:<3} "
        f"{_environment_label(env.path, 30, path_mode=path_mode, base_path=base_path):<30} "
        f"{(env.python_version or '-'): <8} "
        f"{format_size(env.size_bytes):>6} "
        f"{format_age(env.modified):>5}{stale}"
    )


def _size_distribution(
    environments: list[EnvInfo], *, path_mode: PathMode, base_path: Path | None = None
) -> str:
    lines: list[str] = ["SIZE DISTRIBUTION"]
    if not environments:
        lines.append("  (no environments)")
        return "\n".join(lines)

    sized = [env for env in environments if env.size_bytes is not None]
    if not sized:
        lines.append("  (run with --deep to include size data)")
        return "\n".join(lines)

    max_size = max((env.size_bytes or 0) for env in sized)
    for env in sized:
        size = env.size_bytes or 0
        lines.append(
            f"  {bar_chart(size, max_size, width=24)} {_environment_label(env.path, 24, path_mode=path_mode, base_path=base_path):<24} {format_size(size):>6}"
        )
    return "\n".join(lines)


def _artifact_table_header(deep: bool) -> str:
    if deep:
        return f"  {'Category':<22} {'Count':>6} {'Size':>8} {'Safety':>16}"
    return f"  {'Category':<22} {'Count':>6} {'Safety':>16}"


def _artifact_row(summary: ArtifactSummary, *, deep: bool) -> str:
    if deep:
        return (
            f"  {summary.pattern:<22} "
            f"{summary.count:>6} "
            f"{format_size(summary.total_size_bytes):>8} "
            f"{SAFETY_TEXT[summary.safety]:>16}"
        )
    return (
        f"  {summary.pattern:<22} {summary.count:>6} {SAFETY_TEXT[summary.safety]:>16}"
    )


def _artifact_distribution(
    artifact_summary: list[ArtifactSummary], *, deep: bool
) -> str:
    lines = ["SIZE DISTRIBUTION (Artifacts)"]
    if not artifact_summary:
        lines.append("  (no artifacts found)")
        return "\n".join(lines)
    if not deep:
        lines.append("  (run with --deep to include size data)")
        return "\n".join(lines)

    sized = [item for item in artifact_summary if item.total_size_bytes > 0]
    if not sized:
        lines.append("  (no artifact size data)")
        return "\n".join(lines)

    max_size = max(item.total_size_bytes for item in sized)
    for item in sized:
        lines.append(
            f"  {bar_chart(item.total_size_bytes, max_size, width=24)} {item.pattern:<20} {format_size(item.total_size_bytes):>6}"
        )
    return "\n".join(lines)


def format_report(
    result: ScanResult,
    *,
    title: str = "ENVOIC - Python Environment Report",
    path_mode: PathMode = "name",
    deep: bool = False,
    show_artifact_details: bool = False,
) -> str:
    stale_count = sum(1 for env in result.environments if env.is_stale)
    artifact_count = sum(item.count for item in result.artifact_summary)
    artifact_total_size = sum(item.total_size_bytes for item in result.artifact_summary)

    lines: list[str] = []
    lines.append(_box_top())
    lines.append(box_line(f"  {title}"))
    lines.append(box_line("  TR-200  Environment Scanner"))
    lines.append(_box_mid())
    lines.append(_row("Date", result.timestamp.strftime("%Y-%m-%d %H:%M:%S")))
    lines.append(_row("Host", result.hostname))
    lines.append(_row("Scan Path", shorten_path(result.scan_path, 36)))
    lines.append(_row("Scan Depth", str(result.scan_depth)))
    lines.append(_row("Duration", f"{result.duration_seconds:.2f}s"))
    lines.append(_box_mid())
    lines.append(_row("Envs Found", str(len(result.environments))))
    lines.append(
        _row("Env Size", format_size(result.total_size_bytes) if deep else "-")
    )
    lines.append(_row("Stale >90d", str(stale_count)))
    lines.append(_row("Artifacts Found", str(artifact_count)))
    lines.append(
        _row("Artifact Size", format_size(artifact_total_size) if deep else "-")
    )
    lines.append(_box_bottom())
    lines.append("")
    lines.append("ENVIRONMENTS")
    lines.append("─" * 58)
    lines.append(_table_header())
    lines.append("─" * 58)

    if not result.environments:
        lines.append("  (no environments found)")
    else:
        sorted_envs = sorted(result.environments, key=lambda item: str(item.path))
        for index, env in enumerate(sorted_envs, start=1):
            lines.append(
                _table_row(
                    index,
                    env,
                    path_mode=path_mode,
                    base_path=result.scan_path,
                )
            )

    lines.append("─" * 58)
    lines.append("")
    if show_artifact_details:
        lines.append("ARTIFACTS")
        lines.append("─" * 58)
        lines.append(_artifact_table_header(deep))
        lines.append("─" * 58)
        if not result.artifact_summary:
            lines.append("  (no artifacts found)")
        else:
            for summary in result.artifact_summary:
                lines.append(_artifact_row(summary, deep=deep))
        lines.append("─" * 58)
        if deep and any(
            item.safety == SafetyLevel.CAREFUL for item in result.artifact_summary
        ):
            careful_patterns = {
                item.pattern
                for item in result.artifact_summary
                if item.safety == SafetyLevel.CAREFUL
            }
            for pattern in sorted(careful_patterns):
                note = CAREFUL_NOTES.get(pattern)
                if note:
                    lines.append(f"  * {pattern}: {note}")
            lines.append("─" * 58)
        if not deep:
            lines.append("  (run with --deep to include size data)")
            lines.append("─" * 58)
        lines.append("")
        lines.append(
            _artifact_distribution(
                result.artifact_summary,
                deep=deep,
            )
        )
        lines.append("")
    else:
        lines.append("ARTIFACT DETAILS")
        lines.append("─" * 58)
        lines.append(
            "  (hidden by default; run with --artifact to show detailed breakdown)"
        )
        lines.append("─" * 58)
        lines.append("")
    lines.append(
        _size_distribution(
            result.environments, path_mode=path_mode, base_path=result.scan_path
        )
    )

    return "\n".join(lines)


def format_list(
    environments: list[EnvInfo],
    *,
    path_mode: PathMode = "name",
    base_path: Path | None = None,
) -> str:
    lines = [_table_header(), "─" * 58]
    for index, env in enumerate(
        sorted(environments, key=lambda item: str(item.path)), start=1
    ):
        lines.append(_table_row(index, env, path_mode=path_mode, base_path=base_path))
    if len(lines) == 2:
        lines.append("  (no environments found)")
    return "\n".join(lines)


def format_info(env: EnvInfo, top_packages: list[str], activation: str) -> str:
    lines: list[str] = []
    lines.append(_box_top())
    lines.append(box_line("  ENVOIC - Environment Detail"))
    lines.append(_box_mid())
    lines.append(_row("Path", str(env.path)))
    lines.append(_row("Type", env.env_type.value))
    lines.append(_row("Python", env.python_version or "-"))
    lines.append(_row("Size", format_size(env.size_bytes)))
    lines.append(_row("Packages", str(env.package_count or 0)))
    lines.append(
        _row(
            "Modified",
            env.modified.strftime("%Y-%m-%d %H:%M:%S") if env.modified else "-",
        )
    )
    lines.append(
        _row(
            "Created", env.created.strftime("%Y-%m-%d %H:%M:%S") if env.created else "-"
        )
    )
    lines.append(_row("Activate", activation))
    lines.append(_box_bottom())
    lines.append("")
    lines.append("TOP PACKAGES")
    lines.append("─" * 58)
    if not top_packages:
        lines.append("  (no package metadata found)")
    else:
        for idx, name in enumerate(top_packages, start=1):
            lines.append(f"  {idx:>2}. {name}")
    lines.append("─" * 58)
    return "\n".join(lines)


__all__ = [
    "box_line",
    "bar_chart",
    "format_age",
    "format_info",
    "format_list",
    "format_report",
    "format_size",
]



from pathlib import Path

from envoic.scanner import scan


def _touch(path: Path) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text("", encoding="utf-8")


def test_scan_finds_named_and_pyvenv_dirs(tmp_path: Path) -> None:
    named = tmp_path / "project" / ".venv"
    named.mkdir(parents=True)
    _touch(named / "bin" / "activate")

    pyvenv_only = tmp_path / "other" / "custom"
    pyvenv_only.mkdir(parents=True)
    (pyvenv_only / "pyvenv.cfg").write_text("version = 3.11.0\n", encoding="utf-8")

    discovery = scan(tmp_path, max_depth=4)
    found = discovery.environments

    assert named.resolve() in found
    assert pyvenv_only.resolve() in found


def test_scan_respects_depth(tmp_path: Path) -> None:
    deep_env = tmp_path / "a" / "b" / "c" / ".venv"
    deep_env.mkdir(parents=True)
    _touch(deep_env / "bin" / "activate")

    shallow = scan(tmp_path, max_depth=2).environments
    deep = scan(tmp_path, max_depth=5).environments

    assert deep_env.resolve() not in shallow
    assert deep_env.resolve() in deep


def test_scan_skips_git_and_node_modules(tmp_path: Path) -> None:
    git_env = tmp_path / ".git" / ".venv"
    git_env.mkdir(parents=True)
    _touch(git_env / "bin" / "activate")

    node_env = tmp_path / "node_modules" / ".venv"
    node_env.mkdir(parents=True)
    _touch(node_env / "bin" / "activate")

    found = scan(tmp_path, max_depth=5).environments

    assert git_env.resolve() not in found
    assert node_env.resolve() not in found



from pathlib import Path

from envoic.detector import detect_environment
from envoic.models import EnvType


def _touch(path: Path) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text("", encoding="utf-8")


def test_detect_with_pyvenv_cfg(tmp_path: Path) -> None:
    env_dir = tmp_path / ".venv"
    env_dir.mkdir()
    (env_dir / "pyvenv.cfg").write_text("version = 3.12.1\n", encoding="utf-8")

    info = detect_environment(env_dir)

    assert info.env_type == EnvType.VENV
    assert info.has_pyvenv_cfg is True
    assert info.python_version == "3.12.1"
    assert "pyvenv.cfg" in info.signals


def test_detect_with_python_and_site_packages(tmp_path: Path) -> None:
    env_dir = tmp_path / "venv"
    _touch(env_dir / "bin" / "python")
    (env_dir / "lib" / "python3.11" / "site-packages").mkdir(parents=True)

    info = detect_environment(env_dir)

    assert info.env_type == EnvType.VENV
    assert "python+site-packages" in info.signals


def test_detect_conda_environment(tmp_path: Path) -> None:
    env_dir = tmp_path / "conda-env"
    (env_dir / "conda-meta").mkdir(parents=True)

    info = detect_environment(env_dir)

    assert info.env_type == EnvType.CONDA
    assert "conda-meta" in info.signals


def test_detect_plain_dotenv_dir(tmp_path: Path) -> None:
    env_dir = tmp_path / ".env"
    env_dir.mkdir()

    info = detect_environment(env_dir)

    assert info.env_type == EnvType.DOTENV_DIR


def test_detect_unknown_for_non_env(tmp_path: Path) -> None:
    folder = tmp_path / "project"
    folder.mkdir()

    info = detect_environment(folder)

    assert info.env_type == EnvType.UNKNOWN



from __future__ import annotations

import json
import socket
import time
from datetime import UTC, datetime
from pathlib import Path

import typer

from . import __version__
from .artifacts import summarize_artifacts, summarize_with_empty_patterns
from .detector import activation_hint, detect_environment, list_top_packages
from .manager import (
    confirm_careful_artifacts,
    confirm_deletion,
    delete_environments,
    flatten_artifact_summary,
    interactive_select_with_artifacts,
    print_deletion_report,
)
from .models import (
    ArtifactInfo,
    EnvInfo,
    EnvType,
    SafetyLevel,
    ScanResult,
    to_serializable_dict,
)
from .report import PathMode, format_info, format_list, format_report
from .scanner import scan as scan_paths

app = typer.Typer(help="Discover and report Python virtual environments.")


def _build_scan_result(
    path: Path,
    depth: int,
    *,
    deep: bool,
    stale_days: int,
    include_dotenv: bool,
    include_artifacts: bool = True,
) -> ScanResult:
    start = time.perf_counter()
    discovery = scan_paths(
        path,
        max_depth=depth,
        include_artifacts=include_artifacts,
        deep=deep,
    )

    envs: list[EnvInfo] = []
    for candidate in discovery.environments:
        env_info = detect_environment(
            candidate,
            deep=deep,
            stale_days=stale_days,
            include_dotenv=include_dotenv,
        )
        if env_info.env_type == EnvType.UNKNOWN:
            continue
        if env_info.env_type == EnvType.DOTENV_DIR and not include_dotenv:
            continue
        envs.append(env_info)

    duration = time.perf_counter() - start
    total_size_bytes = sum(env.size_bytes or 0 for env in envs)
    artifacts = discovery.artifacts if include_artifacts else []

    return ScanResult(
        scan_path=path.resolve(),
        scan_depth=depth,
        duration_seconds=duration,
        environments=sorted(envs, key=lambda item: str(item.path)),
        total_size_bytes=total_size_bytes,
        hostname=socket.gethostname(),
        timestamp=datetime.now(UTC),
        artifacts=artifacts,
        artifact_summary=summarize_artifacts(artifacts),
    )


def _print_output(text: str, use_rich: bool) -> None:
    if use_rich:
        try:
            from rich.console import Console

            Console().print(text)
            return
        except Exception:
            pass
    typer.echo(text)


def _confirm_and_delete(
    selected: list[EnvInfo | ArtifactInfo],
    *,
    scan_root: Path,
    initial_total: int,
    dry_run: bool,
    yes: bool,
) -> None:
    confirmed = confirm_deletion(
        selected,
        scan_root=scan_root,
        dry_run=dry_run,
        skip_confirm=yes,
    )
    if dry_run:
        summary = delete_environments(
            selected,
            scan_root=scan_root,
            dry_run=True,
            dry_run_echo=False,
        )
        print_deletion_report(summary, initial_total=initial_total)
        raise typer.Exit(0)

    if not confirmed:
        typer.echo("Deletion cancelled.")
        raise typer.Exit(0)

    summary = delete_environments(
        selected,
        scan_root=scan_root,
        dry_run=False,
    )
    print_deletion_report(summary, initial_total=initial_total)


@app.command()
def scan(
    path: Path = typer.Argument(Path("."), exists=True, file_okay=False, dir_okay=True),
    depth: int = typer.Option(5, "--depth", "-d", min=1, help="Max directory depth."),
    deep: bool = typer.Option(
        False, "--deep", help="Compute size and package metadata."
    ),
    json_output: bool = typer.Option(
        False, "--json", help="Output JSON report.", rich_help_panel="Output"
    ),
    stale_days: int = typer.Option(
        90, "--stale-days", min=1, help="Mark env as stale after N days."
    ),
    include_dotenv: bool = typer.Option(
        False, "--include-dotenv", help="Include plain .env directories."
    ),
    include_artifacts: bool = typer.Option(
        True,
        "--artifacts/--no-artifacts",
        help="Include Python artifact detection.",
    ),
    show_artifacts: bool = typer.Option(
        False,
        "--show-artifacts",
        "-a",
        help="Show detailed artifact-level report sections.",
    ),
    path_mode: PathMode = typer.Option(
        "name",
        "--path-mode",
        help="How to render environment path columns: name, relative, absolute.",
    ),
    rich_output: bool = typer.Option(
        False, "--rich", help="Use optional rich-rendered output."
    ),
) -> None:
    """Scan a filesystem path for Python environments."""
    result = _build_scan_result(
        path,
        depth,
        deep=deep,
        stale_days=stale_days,
        include_dotenv=include_dotenv,
        include_artifacts=include_artifacts,
    )

    if json_output:
        typer.echo(json.dumps(to_serializable_dict(result), indent=2))
        raise typer.Exit(0)

    _print_output(
        format_report(
            result,
            path_mode=path_mode,
            deep=deep,
            show_artifact_details=show_artifacts,
        ),
        use_rich=rich_output,
    )


@app.command(name="list")
def list_environments(
    path: Path = typer.Argument(Path("."), exists=True, file_okay=False, dir_okay=True),
    depth: int = typer.Option(5, "--depth", "-d", min=1, help="Max directory depth."),
    deep: bool = typer.Option(
        False, "--deep", help="Compute size and package metadata."
    ),
    stale_days: int = typer.Option(
        90, "--stale-days", min=1, help="Mark env as stale after N days."
    ),
    include_dotenv: bool = typer.Option(
        False, "--include-dotenv", help="Include plain .env directories."
    ),
    path_mode: PathMode = typer.Option(
        "name",
        "--path-mode",
        help="How to render environment path columns: name, relative, absolute.",
    ),
    rich_output: bool = typer.Option(
        False, "--rich", help="Use optional rich-rendered output."
    ),
) -> None:
    """Print a compact environments table."""
    result = _build_scan_result(
        path,
        depth,
        deep=deep,
        stale_days=stale_days,
        include_dotenv=include_dotenv,
        include_artifacts=False,
    )
    _print_output(
        format_list(
            result.environments, path_mode=path_mode, base_path=result.scan_path
        ),
        use_rich=rich_output,
    )


@app.command()
def info(
    env_path: Path = typer.Argument(..., exists=True, file_okay=False, dir_okay=True),
    rich_output: bool = typer.Option(
        False, "--rich", help="Use optional rich-rendered output."
    ),
) -> None:
    """Show detailed information for one environment."""
    env = detect_environment(env_path, deep=True)
    if env.env_type == EnvType.UNKNOWN:
        typer.echo(f"Not a recognized Python environment: {env_path}", err=True)
        raise typer.Exit(code=1)

    top_packages = list_top_packages(env.path, limit=10)
    activation = activation_hint(env.path, env.env_type)
    _print_output(format_info(env, top_packages, activation), use_rich=rich_output)


@app.command()
def version() -> None:
    """Print envoic version."""
    typer.echo(__version__)


@app.command()
def manage(
    path: Path = typer.Argument(Path("."), exists=True, file_okay=False, dir_okay=True),
    depth: int = typer.Option(5, "--depth", "-d", min=1, help="Max directory depth."),
    stale_only: bool = typer.Option(
        False, "--stale-only", help="Pre-select only stale environments."
    ),
    stale_days: int = typer.Option(
        90, "--stale-days", min=1, help="Mark env as stale after N days."
    ),
    dry_run: bool = typer.Option(
        False, "--dry-run", help="Show what would be deleted without deleting."
    ),
    yes: bool = typer.Option(
        False, "--yes", "-y", help="Skip final confirmation (dangerous)."
    ),
    deep: bool = typer.Option(
        False, "--deep", help="Compute size and package metadata for selection view."
    ),
) -> None:
    """Interactively select and delete Python environments."""
    typer.echo(f"Scanning {path.resolve()}...")
    result = _build_scan_result(
        path,
        depth,
        deep=deep,
        stale_days=stale_days,
        include_dotenv=False,
        include_artifacts=True,
    )
    if not result.environments and not result.artifacts:
        typer.echo("No environments or artifacts found.")
        raise typer.Exit(0)

    typer.echo("")
    typer.echo(
        f"Found {len(result.environments)} Python environments and {len(result.artifacts)} artifacts."
    )

    selected_envs: list[EnvInfo] = []
    selected_artifact_groups = []
    artifact_groups = summarize_with_empty_patterns(result.artifacts)
    while True:
        selected_envs, selected_artifact_groups = interactive_select_with_artifacts(
            result.environments,
            artifact_groups,
            scan_root=result.scan_path,
            stale_only=stale_only,
        )
        if not selected_envs and not selected_artifact_groups:
            typer.echo("Nothing selected.")
            raise typer.Exit(0)
        careful_selected = [
            item
            for item in selected_artifact_groups
            if item.safety == SafetyLevel.CAREFUL
        ]
        if not careful_selected:
            break
        if confirm_careful_artifacts(careful_selected):
            break
        typer.echo("")
        typer.echo("Reopening selection...")
        typer.echo("")

    selected_items: list[EnvInfo | ArtifactInfo] = [
        *selected_envs,
        *flatten_artifact_summary(selected_artifact_groups),
    ]

    _confirm_and_delete(
        selected_items,
        scan_root=result.scan_path,
        initial_total=len(result.environments) + len(result.artifacts),
        dry_run=dry_run,
        yes=yes,
    )


@app.command()
def clean(
    path: Path = typer.Argument(Path("."), exists=True, file_okay=False, dir_okay=True),
    depth: int = typer.Option(5, "--depth", "-d", min=1, help="Max directory depth."),
    stale_days: int = typer.Option(
        90, "--stale-days", min=1, help="Delete envs older than N days."
    ),
    dry_run: bool = typer.Option(
        False, "--dry-run", help="Show what would be deleted without deleting."
    ),
    yes: bool = typer.Option(
        False, "--yes", "-y", help="Skip final confirmation (dangerous)."
    ),
    deep: bool = typer.Option(
        True, "--deep/--no-deep", help="Compute size metadata for stale candidates."
    ),
) -> None:
    """Delete stale environments without interactive selection."""
    typer.echo(f"Scanning {path.resolve()} for stale environments...")
    result = _build_scan_result(
        path,
        depth,
        deep=deep,
        stale_days=stale_days,
        include_dotenv=False,
        include_artifacts=False,
    )
    selected = [env for env in result.environments if env.is_stale]
    if not selected:
        typer.echo("No stale environments found.")
        raise typer.Exit(0)

    typer.echo(f"Found {len(selected)} stale environments.")
    _confirm_and_delete(
        selected,
        scan_root=result.scan_path,
        initial_total=len(result.environments),
        dry_run=dry_run,
        yes=yes,
    )


def main() -> None:
    """Console-script entrypoint."""
    app()


if __name__ == "__main__":
    main()



from ._version import __version__
from .models import (
    ArtifactCategory,
    ArtifactInfo,
    ArtifactSummary,
    ArtifactSummaryDict,
    EnvInfo,
    EnvInfoDict,
    EnvType,
    SafetyLevel,
    ScanResult,
    ScanResultDict,
    to_serializable_dict,
)

__all__ = [
    "__version__",
    "ArtifactCategory",
    "ArtifactInfo",
    "ArtifactSummary",
    "ArtifactSummaryDict",
    "EnvInfo",
    "EnvInfoDict",
    "EnvType",
    "SafetyLevel",
    "ScanResult",
    "ScanResultDict",
    "to_serializable_dict",
]



from __future__ import annotations

from pathlib import Path

from envoic.artifacts import summarize_artifacts
from envoic.models import ArtifactCategory, SafetyLevel
from envoic.scanner import scan


def _write_bytes(path: Path, size: int = 16) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_bytes(b"x" * size)


def test_detect_pycache(tmp_path: Path) -> None:
    _write_bytes(tmp_path / "a" / "__pycache__" / "mod.cpython-311.pyc")
    _write_bytes(tmp_path / "a" / "b" / "__pycache__" / "mod.cpython-312.pyc")

    artifacts = scan(tmp_path, max_depth=5, include_artifacts=True).artifacts
    pycache_dirs = [item for item in artifacts if item.pattern_matched == "__pycache__"]
    assert len(pycache_dirs) == 2
    assert all(
        item.category == ArtifactCategory.BYTECODE_CACHE for item in pycache_dirs
    )


def test_detect_tool_caches(tmp_path: Path) -> None:
    (tmp_path / ".mypy_cache").mkdir()
    (tmp_path / ".pytest_cache").mkdir()
    (tmp_path / ".ruff_cache").mkdir()

    artifacts = scan(tmp_path, max_depth=3, include_artifacts=True).artifacts
    patterns = {item.pattern_matched for item in artifacts}
    assert ".mypy_cache" in patterns
    assert ".pytest_cache" in patterns
    assert ".ruff_cache" in patterns
    for item in artifacts:
        if item.pattern_matched in {".mypy_cache", ".pytest_cache", ".ruff_cache"}:
            assert item.category == ArtifactCategory.TOOL_CACHE
            assert item.safety == SafetyLevel.ALWAYS_SAFE


def test_detect_tox_nox(tmp_path: Path) -> None:
    (tmp_path / ".tox").mkdir()
    (tmp_path / ".nox").mkdir()

    artifacts = scan(tmp_path, max_depth=3, include_artifacts=True).artifacts
    by_pattern = {item.pattern_matched: item for item in artifacts}
    assert by_pattern[".tox"].safety == SafetyLevel.CAREFUL
    assert by_pattern[".nox"].safety == SafetyLevel.CAREFUL


def test_build_dist_python_project_only(tmp_path: Path) -> None:
    py_proj = tmp_path / "python_proj"
    py_proj.mkdir()
    (py_proj / "pyproject.toml").write_text("[project]\nname='x'\n", encoding="utf-8")
    (py_proj / "dist").mkdir()

    non_py_proj = tmp_path / "frontend"
    non_py_proj.mkdir()
    (non_py_proj / "dist").mkdir()

    artifacts = scan(tmp_path, max_depth=4, include_artifacts=True).artifacts
    dist_paths = {item.path for item in artifacts if item.pattern_matched == "dist"}
    assert (py_proj / "dist").resolve() in dist_paths
    assert (non_py_proj / "dist").resolve() not in dist_paths


def test_artifacts_inside_venv_excluded(tmp_path: Path) -> None:
    venv = tmp_path / "project" / ".venv"
    venv.mkdir(parents=True)
    (venv / "pyvenv.cfg").write_text("version = 3.12.0\n", encoding="utf-8")
    _write_bytes(venv / "lib" / "python3.12" / "__pycache__" / "x.pyc")

    discovery = scan(tmp_path, max_depth=6, include_artifacts=True)
    artifact_paths = {item.path for item in discovery.artifacts}
    assert (venv / "lib" / "python3.12" / "__pycache__").resolve() not in artifact_paths


def test_egg_info_suffix_match(tmp_path: Path) -> None:
    egg = tmp_path / "pkg" / "mypkg.egg-info"
    egg.mkdir(parents=True)

    artifacts = scan(tmp_path, max_depth=4, include_artifacts=True).artifacts
    egg_infos = [item for item in artifacts if item.pattern_matched == "*.egg-info"]
    assert len(egg_infos) == 1
    assert egg_infos[0].path == egg.resolve()


def test_no_artifacts_flag(tmp_path: Path) -> None:
    (tmp_path / ".mypy_cache").mkdir()
    discovery = scan(tmp_path, max_depth=3, include_artifacts=False)
    assert discovery.artifacts == []


def test_artifact_summary_rollup(tmp_path: Path) -> None:
    for idx in range(5):
        _write_bytes(tmp_path / f"pkg{idx}" / "__pycache__" / "x.pyc", size=8)

    discovery = scan(tmp_path, max_depth=4, include_artifacts=True, deep=True)
    summary = summarize_artifacts(discovery.artifacts)
    pycache = next(item for item in summary if item.pattern == "__pycache__")
    assert pycache.count == 5
    assert pycache.total_size_bytes > 0
